@@ -11,9 +11,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from distutils import spawn
 import logging
 import os
 import re
+import select
 import signal
 import subprocess
 import sys
@@ -62,6 +64,14 @@ class Driver(fixtures.Fixture):
             pid = int(f.read().strip())
         self._kill(pid)
 
+    @staticmethod
+    def find_executable(filename, extra_paths):
+        paths = extra_paths + os.getenv('PATH', os.defpath).split(os.pathsep)
+        for path in paths:
+            loc = spawn.find_executable(filename, path)
+            if loc is not None:
+                return loc
+
     @staticmethod
     def find_config_file(filename):
         # NOTE(sileht): order matter, we first check into virtualenv
@@ -91,7 +101,8 @@ class Driver(fixtures.Fixture):
 
     def _exec(self, command, stdout=False, ignore_failure=False,
               stdin=None, wait_for_line=None, path=[], env=None,
-              allow_debug=True):
+              forbidden_line_after_start=None,
+              allow_debug=True, session=False):
         LOG.debug("executing: %s" % command)
 
         complete_env = {}
@@ -125,7 +136,9 @@ class Driver(fixtures.Fixture):
                 stdin=stdin_fd,
                 stdout=stdout_fd,
                 stderr=subprocess.STDOUT,
-                env=complete_env or None)
+                env=complete_env or None,
+                preexec_fn=os.setsid if session else None
+            )
         except OSError as e:
             raise RuntimeError(
                 "Unable to run command `%s': %s" % (" ".join(command), e))
@@ -148,6 +161,7 @@ class Driver(fixtures.Fixture):
                             % (wait_for_line, b"".join(lines)))
                     break
                 decoded_line = fsdecode(line)
+
                 if wait_for_line and re.search(wait_for_line,
                                                decoded_line):
                     break
@@ -155,6 +169,24 @@ class Driver(fixtures.Fixture):
         else:
             stdout_str = None
 
+        if (stdout or wait_for_line) and forbidden_line_after_start:
+            timeout, forbidden_output = forbidden_line_after_start
+            r, w, x = select.select([c.stdout.fileno()], [], [], timeout)
+            if r:
+                line = c.stdout.readline()
+                self._log_output(app, c.pid, line)
+                lines.append(line)
+                if c.poll() is not None:
+                    # Read the rest if the process is dead, this help debugging
+                    while line:
+                        line = c.stdout.readline()
+                        self._log_output(app, c.pid, line)
+                        lines.append(line)
+                if line and re.search(forbidden_output, fsdecode(line)):
+                    raise RuntimeError(
+                        "Program print a forbidden line: `%s'\nOutput: %s"
+                        % (forbidden_output, b"".join(lines)))
+
         if stdout or wait_for_line or debug:
             # Continue to read
             t = threading.Thread(target=self._read_in_bg,
