@@ -31,6 +31,7 @@ from pifpaf.drivers import etcd
 from pifpaf.drivers import fakes3
 from pifpaf.drivers import gnocchi
 from pifpaf.drivers import influxdb
+from pifpaf.drivers import kafka
 from pifpaf.drivers import keystone
 from pifpaf.drivers import memcached
 from pifpaf.drivers import mongodb
@@ -49,6 +50,7 @@ os.environ["PATH"] = ":".join((
     "/opt/influxdb",
     "/usr/share/elasticsearch/bin",
     "/usr/local/sbin",
+    "/opt/kafka/bin",
 ))
 
 
@@ -369,8 +371,14 @@ class TestDrivers(testtools.TestCase):
     def test_rabbitmq_cluster(self):
         a = self.useFixture(rabbitmq.RabbitMQDriver(cluster=True, port=12345))
         self.assertEqual(
-            "rabbit://%s:%s@localhost:%d,localhost:%d,localhost:%d//" % (
-                a.username, a.password, a.port, a.port + 1, a.port + 2),
+            "rabbit://%(user)s:%(pass)s@localhost:%(port1)d,"
+            "%(user)s:%(pass)s@localhost:%(port2)d,"
+            "%(user)s:%(pass)s@localhost:%(port3)d//" % {
+                "user": a.username,
+                "pass": a.password,
+                "port1": a.port,
+                "port2": a.port + 1,
+                "port3": a.port + 2},
             os.getenv("PIFPAF_URL"))
         self.assertEqual(a.nodename + "-1@localhost",
                          os.getenv("PIFPAF_RABBITMQ_NODENAME"))
@@ -385,6 +393,7 @@ class TestDrivers(testtools.TestCase):
         a.kill_node(a.nodename + "-2@localhost")
         a.stop_node(a.nodename + "-3@localhost")
         a.start_node(a.nodename + "-3@localhost")
+        a.start_node(a.nodename + "-2@localhost")
 
     @testtools.skipUnless(spawn.find_executable("couchdb"),
                           "CouchDB not found")
@@ -395,3 +404,16 @@ class TestDrivers(testtools.TestCase):
                          os.getenv("PIFPAF_URL"))
         r = requests.get("http://localhost:%d/" % port)
         self.assertEqual(r.json()["couchdb"], "Welcome")
+
+    @testtools.skipUnless(spawn.find_executable("kafka-server-start.sh"),
+                          "Kafka not found")
+    def test_kafka(self):
+        a = self.useFixture(kafka.KafkaDriver(port=54321,
+                                              zookeeper_port=12345))
+        self.assertEqual("kafka://localhost:54321",
+                         os.getenv("PIFPAF_URL"))
+        self.assertEqual(12345, a.zookeeper_port)
+        self.assertEqual("54321", os.getenv("PIFPAF_KAFKA_PORT"))
+        self.assertEqual("PLAINTEXT", os.getenv("PIFPAF_KAFKA_PROTOCOL"))
+        self.assertEqual("PLAINTEXT://localhost:54321",
+                         os.getenv("PIFPAF_KAFKA_URL"))
